package at.jku.ssw.graph.test;

import org.junit.Assert;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.ExpectedException;

import at.jku.ssw.Iterator;
import at.jku.ssw.graph.Edge;
import at.jku.ssw.graph.Graph;
import at.jku.ssw.graph.Vertex;
import at.jku.students.graph.Warshall;

public class WarshallTest {

	@Test
	public void test() {
		Graph g = new Graph();
		Vertex a = new Vertex('A', true);
		g.addVertex(a);
		Vertex b = new Vertex('B');
		g.addVertex(b);
		Vertex c = new Vertex('C');
		g.addVertex(c);
		Vertex d = new Vertex('D');
		g.addVertex(d);

		g.addEdge(new Edge(a, b, Edge.Kind.Directed));
		g.addEdge(new Edge(b, c, Edge.Kind.Directed));
		g.addEdge(new Edge(c, d, Edge.Kind.Directed));
		long[] m = Warshall.graphToMatrix(g);

		Assert.assertArrayEquals("Matrix",
				new long[] { //
						0b0010, //
						0b0100, //
						0b1000, //
						0b0000, //
				}, m);

		Warshall.transitiveClosure(m);

		Assert.assertArrayEquals("transitive closure",
				new long[] { //
						0b1110, //
						0b1100, //
						0b1000, //
						0b0000, //
				}, m);

		Graph closure = Warshall.matrixToGraph(g.vertices, m);

		int numEdges = sumEdges(m);
		Assert.assertEquals(numEdges, closure.edges.size());

		boolean[][] processed = new boolean[m.length][m.length];
		Iterator it = closure.edges.iterator();
		while (it.hasNext()) {
			Edge e = (Edge) it.next();
			int fromIdx = closure.vertices.indexOf(e.start);
			int toIdx = closure.vertices.indexOf(e.end);
			Assert.assertTrue(Warshall.isBitSet(m[fromIdx], toIdx));
			Assert.assertFalse(processed[fromIdx][toIdx]);
			processed[fromIdx][toIdx] = true;
		}
	}

	@Rule
	public ExpectedException thrown = ExpectedException.none();

	@Test
	public void testUpper() {

		Graph g = new Graph();
		for (int i = 0; i <= 64; i++) {
			g.addVertex(new Vertex((char) i));
		}
		thrown.expect(IllegalArgumentException.class);
		Warshall.graphToMatrix(g);
	}

	private int sumEdges(long[] m) {
		int sum = 0;
		for (long row : m) {
			sum += Long.bitCount(row);
		}
		return sum;
	}

}
